package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * 考生数量检查
 */
public class ExamStudentCountTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ExamStudentCountTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.EXAM_STUDENT_COUNT.getTitle();
    }

    @Override
    protected void execute() {
        Long examId = SysProperty.EXAM_ID;
        if (examId == null) {
            throw new StatusException("【scan.tool.examId】未配置！");
        }

        // 查询参数：{"examId":null,"subjectCode":"","examNumber":"","name":"","studentCode":"","packageCode":"",
        // "seatNumber":"","campusName":"","examSite":"","examRoom":""}
        Map<String, Object> params = JsonHelper.toMap(SysProperty.MORE_PARAMS, Object.class);
        if (params == null) {
            params = new HashMap<>();
        }
        params.put("examId", examId);

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/count";
        String json = JsonHelper.toJson(params);
        log.info("查询考生数量参数：{}", json);

        String result = HttpHelper.post(url, json);
        log.info("查询考生数量结果：{}", result);
    }

}
