package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import org.apache.commons.io.FileUtils;
import org.apache.poi.UnsupportedFileFormatException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Excel转为txt文件
 */
public class ExcelToTxtTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ExcelToTxtTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.EXCEL_TO_TXT.getTitle();
    }

    @Override
    protected void execute() {
        String fileSuffix = FileHelper.getFileSuffix(SysProperty.DATA_FILE);
        if (!fileSuffix.endsWith(".xls") && !fileSuffix.endsWith(".xlsx")) {
            throw new StatusException("目前仅支持后缀名为“.xls“或”.xlsx”的文件！");
        }

        File excelFile = new File(SysProperty.DATA_FILE);
        if (!excelFile.exists()) {
            throw new StatusException("Excel文件不存在！" + SysProperty.DATA_FILE);
        }

        String txtFilePath = SysProperty.DATA_FILE.replace(fileSuffix, ".txt");
        File txtFile = new File(txtFilePath);
        if (txtFile.exists()) {
            FileUtils.deleteQuietly(txtFile);
        }

        log.warn("***** Excel文件内容的第一行默认跳过不读！！！ *****");

        try {
            EasyExcel.read(excelFile, new AnalysisEventListener<Map<Integer, String>>() {
                private int total = 0;

                private final int batchSize = 10000;

                private List<String> list = new ArrayList<>();

                @Override
                public void invoke(Map<Integer, String> data, AnalysisContext context) {
                    StringBuilder line = new StringBuilder();
                    for (int n = 0; n < data.size(); n++) {
                        String value = data.get(n);
                        line.append(value != null ? value : "");

                        if (n < data.size() - 1) {
                            line.append(SysProperty.TEMPLATE_SEPARATOR);
                        }
                    }
                    list.add(line.toString());
                    total++;

                    if (list.size() >= batchSize) {
                        saveData();
                        list.clear();
                    }
                }

                @Override
                public void doAfterAllAnalysed(AnalysisContext context) {
                    saveData();
                    log.info("共 {} 行，转换完成！{}", total, txtFilePath);
                }

                private void saveData() {
                    FileHelper.writeLines(txtFile, list, true);
                }
            }).sheet().doRead();
        } catch (UnsupportedFileFormatException e) {
            log.error("Excel文件读取失败！" + e.getMessage());
        }
    }

}
