package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.ExamStudent;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import com.fasterxml.jackson.databind.JsonNode;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 考生打包文件和扫描结果导出
 */
public class ObjectiveQuestionExportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ObjectiveQuestionExportTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.OBJECTIVE_QUESTION_EXPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long examId = SysProperty.EXAM_ID;
        if (examId == null) {
            throw new StatusException("【scan.tool.examId】未配置！");
        }

        // 查询参数：{"examId":null,"subjectCode":"","examNumber":"","name":"","studentCode":"","packageCode":"",
        // "seatNumber":"","campusName":"","examSite":"","examRoom":""}
        Map<String, Object> params = JsonHelper.toMap(SysProperty.MORE_PARAMS, Object.class);
        if (params == null) {
            params = new HashMap<>();
        }
        params.put("examId", examId);
        params.put("scanStatus", "SCANNED");
        params.put("withAnswer", true);

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/count";
        String json = JsonHelper.toJson(params);
        String result = HttpHelper.post(url, json);
        log.info("查询考生参数：{}，结果：{}", json, result);

        int total = 0;
        JsonNode props = JsonHelper.getNode(result);
        if (props != null && props.has("count")) {
            total = props.get("count").asInt();
        }
        if (total == 0) {
            log.warn("未查询到相关考生！");
            return;
        }

        // 分批处理
        String filePath = String.format("%s/export-%s.txt", SysProperty.DATA_DIR, System.currentTimeMillis());
        String queryUrl = SysProperty.SCAN_SERVER_URL + "/api/tool/exam/student/find";
        params.put("pageSize", 100);
        int pageNumber = 0;
        int sum = 0;

        while (true) {
            params.put("pageNumber", ++pageNumber);
            String queryJson = JsonHelper.toJson(params);
            String queryResult = HttpHelper.post(queryUrl, queryJson);

            List<ExamStudent> list = JsonHelper.toList(queryResult, ExamStudent.class);
            if (CollectionUtils.isEmpty(list)) {
                break;
            }

            this.export(list, filePath);

            sum += list.size();
            float rate = sum * 100f / total;
            log.info("已处理数：{} 进度：{}%", sum, rate);
        }

        log.info("导出文件：" + filePath);
    }

    private void export(List<ExamStudent> list, String filePath) {
        // OW客观题结果格式：准考证号,科目代码,客观题,卷型
        String template = "%s,%s,%s,%s";

        List<String> lines = new ArrayList<>();
        for (ExamStudent data : list) {
            // 客观题 按“;”分割
            String answers = CollectionUtils.isNotEmpty(data.getAnswer())
                    ? StringUtils.join(data.getAnswer(), ";") : "";

            // 如果没有卷型识别结果默认用1，有的话用实际的识别结果
            String paperType = (StringUtils.isNotEmpty(data.getPaperType())) ? data.getPaperType() : "1";

            lines.add(String.format(template, data.getExamNumber(), data.getSubjectCode(), answers, paperType));
        }

        File file = new File(filePath);
        FileHelper.writeLines(file, lines, true);
    }

}
